@@ -1,6 +1,6 @@
 /*
     fileio
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,9 @@
 #if GLIB_CHECK_VERSION(2,6,0)
 # include <glib/gstdio.h>
 #endif /* GLIB_CHECK_VERSION(2,6,0) */
+#ifdef G_OS_WIN32
+# include <shlwapi.h>
+#endif /* G_OS_WIN32 */
 
 
 struct _FileIO
@@ -58,11 +61,11 @@ fileio_get_absolute_path (const gchar *file,
 #ifdef G_OS_WIN32
   gchar *utf8str, *filename;
   gint i = 0, j, s;
-  DWORD dwSize;             /* フルパスの長さ */
-  HANDLE hFind;             /* 検索ハンドル */
+  DWORD dwSize;             /* ja:フルパスの長さ */
+  HANDLE hFind;             /* ja:検索ハンドル */
 # if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR p, lpszFile;
-  TCHAR szFile[MAX_PATH];   /* フルパス */
+  TCHAR szFile[MAX_PATH];   /* ja:フルパス */
   TCHAR lpszLongName[MAX_PATH];
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 
@@ -72,7 +75,7 @@ fileio_get_absolute_path (const gchar *file,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       LPWSTR p, lpszFile;
-      WCHAR szFile[MAX_PATH];   /* フルパス */
+      WCHAR szFile[MAX_PATH];   /* ja:フルパス */
       WCHAR szLongName[MAX_PATH];
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
@@ -80,8 +83,41 @@ fileio_get_absolute_path (const gchar *file,
       utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       if (!utf8str)
         return g_strdup (file);
-      lpszFile = g_utf8_to_utf16 (file, -1, NULL, NULL, NULL);
+      lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       g_free (utf8str);
+      if (PathIsRelativeW (lpszFile))
+        {
+          gint leng;
+          LPWSTR lpszPath = NULL, lpszTemp;
+
+          if (path)
+            {
+              utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+              if (utf8str)
+                {
+                  lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+                  g_free (utf8str);
+                }
+            }
+          if (!lpszPath || PathIsRelativeW (lpszPath))
+            {
+              g_free (lpszPath);
+              lpszPath = g_malloc (MAX_PATH * sizeof (WCHAR));
+              GetCurrentDirectoryW (MAX_PATH, lpszPath);
+            }
+          leng = lstrlenW (lpszPath);
+          if (leng > 0 && G_IS_DIR_SEPARATOR (lpszPath[leng - 1]))
+            lpszPath[--leng] = '\0';
+          lpszTemp = g_malloc ((leng + lstrlenW (lpszFile) + 2)
+                                                            * sizeof (WCHAR));
+          lstrcpyW (lpszTemp, lpszPath);
+          lpszTemp[leng] = G_DIR_SEPARATOR;
+          lpszTemp[leng + 1] = '\0';
+          lstrcatW (lpszTemp, lpszFile);
+          g_free (lpszPath);
+          g_free (lpszFile);
+          lpszFile = lpszTemp;
+        }
       dwSize = GetFullPathNameW (lpszFile, MAX_PATH, szFile, &p);
       g_free (lpszFile);
       if (dwSize == 0 || dwSize >= MAX_PATH)
@@ -156,7 +192,7 @@ fileio_get_absolute_path (const gchar *file,
   else
     {
       LPSTR p, lpszFile;
-      CHAR szFile[MAX_PATH];   /* フルパス */
+      CHAR szFile[MAX_PATH];   /* ja:フルパス */
       CHAR szLongName[MAX_PATH];
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
@@ -168,6 +204,31 @@ fileio_get_absolute_path (const gchar *file,
 #  else /* not GLIB_CHECK_VERSION(2,6,0) */
       lpszFile = g_strdup (file);
 #  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+      if (PathIsRelativeA (lpszFile))
+        {
+          LPSTR lpszPath = NULL, lpszTemp;
+
+          if (path)
+            {
+#  if GLIB_CHECK_VERSION(2,8,0)
+              lpszPath = g_win32_locale_filename_from_utf8 (path);
+#  elif GLIB_CHECK_VERSION(2,6,0)
+              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
+#  else /* not GLIB_CHECK_VERSION(2,6,0) */
+              lpszPath = g_strdup (path);
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+            }
+          if (!lpszPath || PathIsRelativeA (lpszPath))
+            {
+              g_free (lpszPath);
+              lpszPath = g_malloc (MAX_PATH * sizeof (CHAR));
+              GetCurrentDirectoryA (MAX_PATH, lpszPath);
+            }
+          lpszTemp = g_build_filename (lpszPath, lpszFile, NULL);
+          g_free (lpszPath);
+          g_free (lpszFile);
+          lpszFile = lpszTemp;
+        }
       dwSize = GetFullPathNameA (lpszFile, MAX_PATH, szFile, &p);
       g_free (lpszFile);
       if (dwSize == 0 || dwSize >= MAX_PATH)
@@ -239,19 +300,23 @@ fileio_get_absolute_path (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
 # if GLIB_CHECK_VERSION(2,6,0)
     }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
   if (!utf8str)
     return g_strdup (file);
   filename = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
   g_free (utf8str);
   return filename ? filename : g_strdup (file);
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
   gchar *absolate;
   gint i, leng;
 
   if (!file)
     return NULL;
-  if (!G_IS_DIR_SEPARATOR (file[0]))
+  if (g_path_is_absolute (file))
+    {
+      absolate = g_strdup (file);
+    }
+  else
     {
       if (path && g_path_is_absolute (path))
         {
@@ -269,10 +334,6 @@ fileio_get_absolute_path (const gchar *file,
           g_free (dir);
         }
     }
-  else
-    {
-      absolate = g_strdup (file);
-    }
   leng = g_strlen (absolate);
   i = 0;
   while (i < leng)
@@ -289,10 +350,10 @@ fileio_get_absolute_path (const gchar *file,
   i = 0;
   while (i < leng)
     if (G_IS_DIR_SEPARATOR (absolate[i]) && absolate[i + 1] == '.'
-                                    && G_IS_DIR_SEPARATOR (absolate[i + 2]))
+        && (G_IS_DIR_SEPARATOR (absolate[i + 2]) || absolate[i + 2] == '\0'))
       {
         leng -= 2;
-        g_memmove (absolate + i + 1, absolate + i + 3, leng - i);
+        g_memmove (absolate + i, absolate + i + 2, leng - i + 1);
       }
     else
       {
@@ -301,18 +362,19 @@ fileio_get_absolute_path (const gchar *file,
   i = 0;
   while (i < leng)
     if (G_IS_DIR_SEPARATOR (absolate[i]) && absolate[i + 1] == '.'
-            && absolate[i + 2] == '.' && G_IS_DIR_SEPARATOR (absolate[i + 3]))
+        && absolate[i + 2] == '.'
+        && (G_IS_DIR_SEPARATOR (absolate[i + 3])  || absolate[i + 3] == '\0'))
       {
         gint j;
 
         leng -= 3;
-        g_memmove (absolate + i + 1, absolate + i + 4, leng - i);
+        g_memmove (absolate + i, absolate + i + 3, leng - i + 1);
         for (j = i - 1; j >= 0; j--)
           if (G_IS_DIR_SEPARATOR (absolate[j]))
             break;
         if (j >= 0)
           {
-            g_memmove (absolate + j + 1, absolate + i + 1, leng - i);
+            g_memmove (absolate + j, absolate + i, leng - i + 1);
             leng -= i - j;
             i = j;
           }
@@ -321,6 +383,11 @@ fileio_get_absolute_path (const gchar *file,
       {
         i++;
       }
+  for (i = 0; i < leng; i++)
+    if (G_IS_DIR_SEPARATOR (absolate[i]))
+      absolate[i] = G_DIR_SEPARATOR;
+  if (leng <= 0)
+    g_strcpy (absolate, G_DIR_SEPARATOR_S);
   return absolate;
 #endif /* not G_OS_WIN32 */
 }
@@ -334,7 +401,7 @@ gchar *
 fileio_get_relative_path (const gchar *file,
                           const gchar *path)
 {
-  gchar *absolate, *relative = NULL, *name, *tmp;
+  gchar *absolate, *relative = NULL, *name;
   gchar *path_to, *path_from, **ary_to, **ary_from;
   gint i, j;
 
@@ -371,6 +438,8 @@ fileio_get_relative_path (const gchar *file,
   for (j = i; ary_from[j]; j++)
     if (*(ary_from[j]) != '\0')
       {
+        gchar *tmp;
+
         tmp = relative ? g_strconcat (relative, ".."G_DIR_SEPARATOR_S, NULL)
                        : g_strdup (".."G_DIR_SEPARATOR_S);
         g_free (relative);
@@ -378,6 +447,8 @@ fileio_get_relative_path (const gchar *file,
       }
   for (j = i; ary_to[j]; j++)
     {
+      gchar *tmp;
+
       tmp = relative
                 ? g_strconcat (relative, ary_to[j], G_DIR_SEPARATOR_S, NULL)
                 : g_strconcat (ary_to[j], G_DIR_SEPARATOR_S, NULL);
@@ -389,6 +460,8 @@ fileio_get_relative_path (const gchar *file,
   name = g_path_get_basename (file);
   if (name)
     {
+      gchar *tmp;
+
       tmp = relative ? g_strconcat (relative, name, NULL) : g_strdup (name);
       g_free (relative);
       relative = tmp;
@@ -420,9 +493,9 @@ static volatile gboolean critical = FALSE;
        RET,ファイルポインタ,NULL:エラー                                     */
 FileIO *
 fileio_open (const gchar *file,
-             const guint access,
-             const guint share,
-             const guint mode)
+             const guint  access,
+             const guint  share,
+             const guint  mode)
 {
   gchar *path;
   FileIO *fio;
@@ -813,6 +886,40 @@ fileio_size (FileIO *fio)
 }
 
 
+/*  ja:ファイルの終端を設定する
+    fio,ファイルポインタ
+    RET,TRUE:正常終了,FALSE:エラー                                          */
+gboolean
+fileio_eof (FileIO *fio)
+{
+#ifdef G_OS_WIN32
+  return SetEndOfFile (fio->hFile);
+#else /* not G_OS_WIN32 */
+# ifdef HAVE_FTRUNCATE
+  gboolean result;
+  goffset offset;
+
+  offset = fileio_seek (fio, 0, FILEIO_SEEK_CUR);
+  if (offset < 0)
+    return FALSE;
+#  ifdef USE_THREAD
+  G_LOCK (critical);
+  critical = TRUE;
+#  endif /* USE_THREAD */
+  result = fio && g_list_find (glist_fileio, fio)
+                                    ? ftruncate (fio->fd, offset) == 0: FALSE;
+#  ifdef USE_THREAD
+  critical = FALSE;
+  G_UNLOCK (critical);
+#  endif /* USE_THREAD */
+  return result;
+# else /* not HAVE_FTRUNCATE */
+  return FALSE;
+# endif /* not HAVE_FTRUNCATE */
+#endif /* not G_OS_WIN32 */
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:高レベルファイル入出力関数群                                             *
@@ -975,11 +1082,7 @@ fileio_mkdir (const gchar *dir)
   LPTSTR lpszPath;
 #endif /* ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
 
-  if (!dir)
-    return FALSE;
   path = fileio_get_full_path (dir);
-  if (!path)
-    return FALSE;
   parent = g_path_get_dirname (path);
   if (!parent)
     {
@@ -1018,11 +1121,6 @@ fileio_mkdir (const gchar *dir)
 
 # if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
       utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        {
-          g_free (path);
-          return FALSE;
-        }
       lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       g_free (utf8str);
       result = CreateDirectoryW (lpszPath, NULL);
@@ -1063,6 +1161,140 @@ fileio_mkdir (const gchar *dir)
 }
 
 
+/*  ja:ファイル/ディレクトリを削除する
+    path,ファイル/ディレクトリ名
+     RET,TRUE:正常終了,FALSE,エラー                                         */
+gboolean
+fileio_remove (const gchar *path)
+{
+  gboolean result = TRUE;
+
+  if (g_file_test (path, G_FILE_TEST_IS_DIR))
+    {
+      const gchar *name;
+      GDir *dir;
+
+      dir = g_dir_open (path, 0, NULL);
+      if (!dir)
+        return FALSE;
+      while (result && (name = g_dir_read_name (dir)))
+        {
+          gchar *file;
+
+          file = g_build_filename (path, name, NULL);
+          result = fileio_remove (file);
+          g_free (file);
+        }
+      g_dir_close (dir);
+      if (result)
+        {
+#ifdef G_OS_WIN32
+# if ! GLIB_CHECK_VERSION(2,6,0)
+#  ifdef UNICODE
+          gchar *utf8str;
+#  endif /* UNICODE */
+          LPTSTR lpszPath;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# if GLIB_CHECK_VERSION(2,6,0)
+          if (G_WIN32_HAVE_WIDECHAR_API ())
+            {
+              gchar *utf8str;
+              LPWSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
+              utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+              lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+              g_free (utf8str);
+              result = RemoveDirectoryW (lpszPath);
+              g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+            }
+          else
+            {
+              LPSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
+#  if GLIB_CHECK_VERSION(2,8,0)
+              lpszPath = g_win32_locale_filename_from_utf8 (path);
+#  elif GLIB_CHECK_VERSION(2,6,0)
+              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
+#  else /* not GLIB_CHECK_VERSION(2,6,0) */
+              lpszPath = g_strdup (path);
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+              result = RemoveDirectoryA (lpszPath);
+              g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+            }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+#else /* not G_OS_WIN32 */
+# if GLIB_CHECK_VERSION(2,6,0)
+          result = g_rmdir (path) == 0;
+# else /* not GLIB_CHECK_VERSION(2,6,0) */
+          result = rmdir (path) == 0;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#endif /* not G_OS_WIN32 */
+        }
+    }
+  else
+    {
+#ifdef G_OS_WIN32
+# if ! GLIB_CHECK_VERSION(2,6,0)
+#  ifdef UNICODE
+      gchar *utf8str;
+#  endif /* UNICODE */
+      LPTSTR lpszPath;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# if GLIB_CHECK_VERSION(2,6,0)
+      if (G_WIN32_HAVE_WIDECHAR_API ())
+        {
+          gchar *utf8str;
+          LPWSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
+          utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+          lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+          g_free (utf8str);
+          result = DeleteFileW (lpszPath);
+          g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+        }
+      else
+        {
+          LPSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
+#  if GLIB_CHECK_VERSION(2,8,0)
+          lpszPath = g_win32_locale_filename_from_utf8 (path);
+#  elif GLIB_CHECK_VERSION(2,6,0)
+          lpszPath = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
+#  else /* not GLIB_CHECK_VERSION(2,6,0) */
+          lpszPath = g_strdup (file);
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+          result = DeleteFileA (lpszPath);
+          g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+        }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+#else /* not G_OS_WIN32 */
+# if GLIB_CHECK_VERSION(2,6,0)
+      result = g_remove (path) == 0;
+# else /* not GLIB_CHECK_VERSION(2,6,0) */
+      result = remove (path) == 0;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#endif /* not G_OS_WIN32 */
+    }
+  return result;
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:ファイル時刻関数群                                                       *
@@ -1290,16 +1522,21 @@ fileio_getsystime (FileIOTime    *ftime,
 ******************************************************************************/
 /*  ja:ファイルのアイコンを取得する
          file,ファイル名
-    icon_size,FILEIO_ICON_SIZE_SMALL/FILEIO_ICON_SIZE_LARGE
+    icon_size,アイコンサイズ
           RET,pixbuf,NULL:エラー                                            */
 GdkPixbuf *
 fileio_extract_icon (const gchar *file,
-                     const guint  icon_size)
+                     GtkIconSize  icon_size)
 {
 #if defined (G_OS_WIN32)
+  gint icon_x, icon_y;
   GdkPixbuf *pixbuf = NULL;
   HICON hIcon = NULL;
   ICONINFO ii;
+  UINT uFlags;
+# if GLIB_CHECK_VERSION(2,2,0)
+  GtkSettings *settings;
+# endif /* GLIB_CHECK_VERSION(2,2,0) */
 # if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
   SHFILEINFO shfi;
@@ -1310,6 +1547,34 @@ fileio_extract_icon (const gchar *file,
 
   if (!file)
     return NULL;
+  switch (icon_size)
+    {
+      case GTK_ICON_SIZE_MENU:          icon_x = 16; icon_y = 16; break;
+      case GTK_ICON_SIZE_SMALL_TOOLBAR: icon_x = 18; icon_y = 18; break;
+      case GTK_ICON_SIZE_LARGE_TOOLBAR: icon_x = 24; icon_y = 24; break;
+      case GTK_ICON_SIZE_BUTTON:        icon_x = 20; icon_y = 20; break;
+      case GTK_ICON_SIZE_DND:           icon_x = 32; icon_y = 32; break;
+      case GTK_ICON_SIZE_DIALOG:        icon_x = 48; icon_y = 48; break;
+      default:                          icon_x =  0; icon_y =  0;
+    }
+# if GLIB_CHECK_VERSION(2,2,0)
+  settings = gtk_settings_get_default ();
+  if (settings)
+    {
+      gint w, h;
+
+      if (gtk_icon_size_lookup_for_settings (settings, icon_size, &w, &h))
+        {
+          icon_x = w;
+          icon_y = h;
+        }
+    }
+  uFlags = ABS (GetSystemMetrics (SM_CXSMICON) - icon_x)
+                                + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
+                                < ABS (GetSystemMetrics (SM_CXICON) - icon_x)
+                                + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
+                                        ? SHGFI_SMALLICON : SHGFI_LARGEICON;
+# endif /* GLIB_CHECK_VERSION(2,2,0) */
 # if GLIB_CHECK_VERSION(2,6,0)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
@@ -1324,9 +1589,7 @@ fileio_extract_icon (const gchar *file,
         return NULL;
       lpszFile = g_utf8_to_utf16 (file, -1, NULL, NULL, NULL);
       if ((BOOL)SHGetFileInfoW (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
-                                        SHGFI_ICON | SHGFI_USEFILEATTRIBUTES
-                                        | (icon_size == FILEIO_ICON_SIZE_SMALL
-                                        ? SHGFI_SMALLICON : SHGFI_LARGEICON)))
+                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
         hIcon = shfi.hIcon;
       g_free (lpszFile);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
@@ -1347,13 +1610,13 @@ fileio_extract_icon (const gchar *file,
       lpszFile = g_strdup (file);
 #  endif /* not GLIB_CHECK_VERSION(2,8,0) */
       if ((BOOL)SHGetFileInfoA (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
-                                        SHGFI_ICON | SHGFI_USEFILEATTRIBUTES
-                                        | (icon_size == FILEIO_ICON_SIZE_SMALL
-                                        ? SHGFI_SMALLICON : SHGFI_LARGEICON)))
+                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
         hIcon = shfi.hIcon;
       g_free (lpszFile);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
     }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
   if (GetIconInfo (hIcon, &ii))
     {
       BITMAPINFOHEADER bmih;
@@ -1394,7 +1657,7 @@ fileio_extract_icon (const gchar *file,
                     alpha = TRUE;
                   pixels += 4;
                 }
-              pixels += (w * 4 - rowstride);
+              pixels += rowstride - w * 4;
             }
           /* en:mask */
           if (!alpha)
@@ -1409,7 +1672,7 @@ fileio_extract_icon (const gchar *file,
                       pixels[3] = 255 - lpbBits[(x + y * w) * 4];
                       pixels += 4;
                     }
-                  pixels += (w * 4 - rowstride);
+                  pixels += rowstride - w * 4;
                 }
             }
           g_free (lpbBits);
@@ -1418,13 +1681,31 @@ fileio_extract_icon (const gchar *file,
       DeleteObject (ii.hbmColor);
       DeleteObject (ii.hbmMask);
     }
+  if (hIcon)
+    DestroyIcon (hIcon);
   return pixbuf;
 #elif GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   GdkPixbuf *pixbuf = NULL;
 
   if (!file)
     return NULL;
-  if (!g_file_test (file, G_FILE_TEST_IS_DIR))
+  if (g_file_test (file, G_FILE_TEST_IS_DIR))
+    {
+      const gchar *home;
+      gint leng;
+      GtkWidget *image;
+
+      home = g_get_home_dir ();
+      leng = g_strlen (home);
+      image = gtk_image_new ();
+      pixbuf = gtk_widget_render_icon (image,
+            g_strfilencmp (file, home, leng) == 0
+            && (file[leng] == '\0'
+                || (G_IS_DIR_SEPARATOR (file[leng]) && file[leng + 1] == '\0'))
+                    ? GTK_STOCK_HOME : GTK_STOCK_DIRECTORY, icon_size, NULL);
+      gtk_widget_destroy (image);
+    }
+  else
     {
       gchar *text;
       gssize length;
@@ -1487,55 +1768,76 @@ fileio_extract_icon (const gchar *file,
                                                                     icon_name);
                         }
                       if (result)
-# if GLIB_CHECK_VERSION(2,6,0)
                         {
+                          gint icon_x, icon_y;
+# if GLIB_CHECK_VERSION(2,6,0)
                           gint *sizes;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+                          GtkSettings *settings;
 
+                          switch (icon_size)
+                            {
+                              case GTK_ICON_SIZE_MENU:
+                                icon_x = icon_y = 16;
+                                break;
+                              case GTK_ICON_SIZE_SMALL_TOOLBAR:
+                                icon_x = icon_y = 18;
+                                break;
+                              case GTK_ICON_SIZE_LARGE_TOOLBAR:
+                                icon_x = icon_y = 24;
+                                break;
+                              case GTK_ICON_SIZE_BUTTON:
+                                icon_x = icon_y = 20;
+                                break;
+                              case GTK_ICON_SIZE_DND:
+                                icon_x = icon_y = 32;
+                                break;
+                              case GTK_ICON_SIZE_DIALOG:
+                                icon_x = icon_y = 48;
+                                break;
+                              default:
+                                icon_x = icon_y = 0;
+                            }
+                          settings = gtk_settings_get_default ();
+                          if (settings)
+                            {
+                              gint w, h;
+
+                              if (gtk_icon_size_lookup_for_settings (settings,
+                                                            icon_size, &w, &h))
+                                {
+                                  icon_x = w;
+                                  icon_y = h;
+                                }
+                            }
+# if GLIB_CHECK_VERSION(2,6,0)
                           sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
                                                                     icon_name);
                           if (sizes)
                             {
-                              gint n = 0, s;
+                              gint n = 0;
 
-                              s = icon_size == FILEIO_ICON_SIZE_SMALL
-                                                                    ? 16 : 32;
                               for (i = 0; sizes[i] > 0; i++)
-                                if (ABS (s - sizes[i]) < ABS (s - n))
+                                if (ABS (icon_x - sizes[i])
+                                                    + ABS (icon_y - sizes[i])
+                                        < ABS (icon_x - n) + ABS (icon_y - n))
                                   n = sizes[i];
                               g_free (sizes);
                               if (n > 0)
                                 pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                                         icon_name, n, 0, NULL);
                             }
+                          if (!pixbuf)
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+                            pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                    icon_name, (icon_x + icon_y) / 2, 0, NULL);
                         }
-# else /* not GLIB_CHECK_VERSION(2,6,0) */
-                        pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                            icon_name,
-                            icon_size == FILEIO_ICON_SIZE_SMALL ? 16 : 32,
-                            0, NULL);
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
                     }
                 }
               g_free (icon_name);
             }
         }
     }
-# if 0
-  if (!pixbuf)
-    {
-      GtkWidget *image;
-
-      image = gtk_image_new ();
-      pixbuf = gtk_widget_render_icon (image,
-                                    g_file_test (file, G_FILE_TEST_IS_DIR)
-                                        ? GTK_STOCK_DIRECTORY : GTK_STOCK_FILE,
-                                    icon_size == FILEIO_ICON_SIZE_SMALL
-                                        ? GTK_ICON_SIZE_SMALL_TOOLBAR
-                                        : GTK_ICON_SIZE_LARGE_TOOLBAR,
-                                    NULL);
-      gtk_widget_destroy (image);
-    }
-# endif
   return pixbuf;
 #else /* not G_OS_WIN32 && not MIMEFILE */
   return NULL;
