@@ -1,6 +1,6 @@
 /*
     winmain
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include <windows.h>
-#include "common.h"
+#include "gcommon.h"
 
 
 int
@@ -34,7 +34,7 @@ WinMain (HINSTANCE hInstance,
   char **argv;
   int i, argc, ret = -1;
 
-  if (G_WIN32_HAVE_WIDECHAR_API())
+  if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       int nArgc;
       LPWSTR *lpszArgv;
