@@ -84,14 +84,15 @@ orz_winmenu_get_type (void) G_GNUC_CONST;
 ******************************************************************************/
 /*  ja:新規作成
     RET,オブジェクト                                                        */
-GObject*
+GObject *
 orz_winmenu_new (void);
 
 
 /*  ja:MDIを取得する
     winmenu,オブジェクト
-       mdi,MDI                                                              */
-#define orz_winmenu_get_mdi(winmenu) ((winmenu)?ORZ_WINMENU(winmenu)->mdi:NULL)
+        RET,MDI                                                             */
+OrzMdi *
+orz_winmenu_get_mdi (OrzWinmenu *winmenu);
 
 
 /*  ja:MDIを設定する
@@ -104,9 +105,9 @@ orz_winmenu_set_mdi (OrzWinmenu *winmenu,
 
 /*  ja:メニューを取得する
     winmenu,オブジェクト
-       menu,メニュー                                                        */
-#define orz_winmenu_get_menu(winmenu)                                       \
-                            ((winmenu)?ORZ_WINMENU(winmenu)->menu_shell:NULL)
+        RET,メニュー                                                        */
+GtkWidget *
+orz_winmenu_get_menu (OrzWinmenu *winmenu);
 
 
 /*  ja:メニューを設定する
