@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -1146,7 +1146,7 @@ property_dialog (GtkWindow *parent,
 
           text = gtk_entry_get_text
                                 (GTK_ENTRY (GTK_COMBO (propdlg.combo)->entry));
-          glist = gtk_container_children
+          glist = gtk_container_get_children
                             (GTK_CONTAINER (GTK_COMBO (propdlg.combo)->list));
           for (ft->crlf = g_list_length (glist) - 1; ft->crlf >= 0; ft->crlf--)
             if (g_strcmp (gtk_label_get_text (GTK_LABEL (gtk_bin_get_child
